#include "StdAfx.h"
#include "ComboMenuStrip.h"
using namespace System::Windows::Forms;
using namespace System;
#define WM_MOUSEMOVE                    0x0200
#define WM_MOUSELEAVE                   0x02A3

/******************************************************************
ComboMenuStrip: CONSTRUCTOR/DESTRUCTOR
*******************************************************************/

ComboMenuStrip::ComboMenuStrip()
{
  tool_tip = gcnew System::Windows::Forms::ToolTip();
  this->Cursor=System::Windows::Forms::Cursors::Arrow;
  setBaseItemOnClick_ = true;
  InitializeComponent();
  //
  //TODO: Add the constructor code here
  //
}

ComboMenuStrip::~ComboMenuStrip()
{
  if (components)
  {
    delete components;
  }
}

/******************************************************************
ComboMenuStrip: PRIVATE FUNCTIONS
*******************************************************************/
void 
ComboMenuStrip::InitializeComponent()  {
}

/******************************************************************
ComboMenuStrip: EVENT HANDLERS
*******************************************************************/

//handles the click event on menu items to be added to the control
//extracts the name and UniqueTag from the item clicked and sets
//this in the base menu item
void 
ComboMenuStrip::clickMenu(System::Object^ sender, System::EventArgs ^e) {

  //cast the sender object
  MenuItemStoredValue^ menu_item_clicked=dynamic_cast<MenuItemStoredValue^>(sender);


  //set the text and variable in the top level menu item if required. Default behavior is to 
  //set menu items but when using control in a table this can be handled elsewhere
  if(setBaseItemOnClick_) {
    MenuItemStoredValue^ base_menu_item = getBaseMenuItem();
    if(base_menu_item != nullptr)  {
      base_menu_item->setItemStored(menu_item_clicked->getItemStored());
      base_menu_item->Text=menu_item_clicked->getDisplayName();
      base_menu_item->setDisplayName(menu_item_clicked->getDisplayName());
    }
  }

  //raise the selected event
  menuSelected(this,gcnew MenuItemSelectedEventArgs(menu_item_clicked->getDisplayName(),menu_item_clicked->getItemStored()));
}

/******************************************************************
ComboMenuStrip: PUBLIC FUNCTIONS
*******************************************************************/

//reset the base menu item in the control
void 
ComboMenuStrip::setBaseMenuItem(MenuItemStoredValue^ base_menu_item) {

  //clear the items from the menu strip and add the base menu item with 
  this->Items->Clear();
  this->Items->Add(base_menu_item);
}

MenuItemStoredValue^ 
ComboMenuStrip::getBaseMenuItem()  {

  if(this->Items->Count > 0)  {
    return dynamic_cast<MenuItemStoredValue^>(this->Items[0]);
  }
  else  {
    return nullptr;
  }
}

//differs from setBaseMenuItem in that only the top level menu_item is changed and the tree is preserved
void 
ComboMenuStrip::setSelection(MenuItemStoredValue^ menu_item)  {
  MenuItemStoredValue^ base_menu_item = getBaseMenuItem();

  if(base_menu_item != nullptr)  {

    base_menu_item->setItemStored(menu_item->getItemStored());
    base_menu_item->setDisplayName(menu_item->getDisplayName());
    base_menu_item->Text = menu_item->getDisplayName();
  }
}

void 
ComboMenuStrip::WndProc(System::Windows::Forms::Message %p_message)
{
  //mouse move event - set the tooltip and do not call the base class
  //this stops highlighting of the active button
  if(p_message.Msg == WM_MOUSEMOVE)
  {
    if(tool_tip->GetToolTip(this)->Length == 0)
    {
      //set the tooltip to the base menu item it this exists
      if(this->Items->Count>0)
      {
        MenuItemStoredValue^ base_menu_item=dynamic_cast<MenuItemStoredValue^>(this->Items[0]);
        if(base_menu_item)
        {
          tool_tip->SetToolTip(this,base_menu_item->getDisplayName());
        }
      }
    }
    return;
  }
  else if(p_message.Msg == WM_MOUSELEAVE)
  {
    tool_tip->RemoveAll();
  }

  __super::WndProc(p_message);

}


void
ComboMenuStrip::OnClick(EventArgs ^e) {



  __super::OnClick(e);
}




void
ComboMenuStrip::OnPaint(PaintEventArgs ^e) {

  //__super::OnPaint(e);



  Drawing::Graphics  ^graphics  = e->Graphics;
  Drawing::Rectangle  r         = ClientRectangle;

  unsigned char border       = 4;

  Drawing::SolidBrush  ^center_color_brush;
  Drawing::Pen         ^pen;

  //add a 3-D border around the control 
  Drawing::Rectangle  border_rect         = ClientRectangle;
  ControlPaint::DrawBorder3D( e->Graphics, border_rect, Border3DStyle::Raised );

  if (Enabled) {
    center_color_brush = gcnew Drawing::SolidBrush(Drawing::Color::Black);
    pen = gcnew Drawing::Pen(Drawing::SystemColors::WindowText);
  }
  else {
    center_color_brush = gcnew Drawing::SolidBrush(Drawing::SystemColors::GrayText);
    pen = gcnew Drawing::Pen(Drawing::SystemColors::ControlDark);
  }

  //if there is a base menu item present then draw the text in the control 
  MenuItemStoredValue^ base_menu = getBaseMenuItem();
  if(base_menu != nullptr)  {
    System::String^ draw_text = base_menu->Text;

    Drawing::StringFormat ^format  = gcnew Drawing::StringFormat(Drawing::StringFormatFlags::NoWrap);
    format->LineAlignment = Drawing::StringAlignment::Center;
    format->Alignment     = Drawing::StringAlignment::Near;

    Drawing::RectangleF string_rect(Drawing::RectangleF::FromLTRB(r.Left+border, r.Top+border, r.Right-15, r.Bottom-border));
    graphics->DrawString(draw_text, this->Font, center_color_brush, string_rect, format);

  }

  //draw the arrow
  Drawing::Point  p1  = Drawing::Point(r.Width - 9, r.Height / 2 - 1);
  Drawing::Point  p2  = Drawing::Point(r.Width - 5, r.Height / 2 - 1);
  graphics->DrawLine(pen, p1, p2);

  p1 = Drawing::Point(r.Width - 8, r.Height / 2);
  p2 = Drawing::Point(r.Width - 6, r.Height / 2);
  graphics->DrawLine(pen, p1, p2);

  p1 = Drawing::Point(r.Width - 7, r.Height / 2);
  p2 = Drawing::Point(r.Width - 7, r.Height / 2 + 1);
  graphics->DrawLine(pen, p1, p2);

  //draw the divider line
  pen = gcnew Drawing::Pen(Drawing::SystemColors::ControlDark);
  p1 = Drawing::Point(r.Width - 12, 4);
  p2 = Drawing::Point(r.Width - 12, r.Height - 5);
  graphics->DrawLine(pen, p1, p2);

  pen = gcnew Drawing::Pen(Drawing::SystemColors::ControlLightLight);
  p1 = Drawing::Point(r.Width - 11, 4);
  p2 = Drawing::Point(r.Width - 11, r.Height - 5);
  graphics->DrawLine(pen, p1, p2);

  if(base_menu != nullptr)
    System::Diagnostics::Debug::Write("painting the combo box control " + base_menu->Text + "\n");
}
